function PageItem(name) {
    this.commandName = name
}

// this method is passed the event
PageItem.prototype.OnClick = function(e) {
    var target;

    //alert(DWRUtil.toDescriptiveString(e,2,1));
    if (!e.target) {
        // IE event model
        target = e.srcElement;
    }
    else {
        target = e.target;
    }
    this.selectNode(target);
}


// this method is passed the tag which has been clicked on
PageItem.prototype.OnDoubleClick = function( e ) {
    this.selectNode(e);
}

PageItem.prototype.findTarget = function(t) {
    if (t == null)
        return null;

    if (this.isOurNode(t))
        return t;

    return this.findTarget(t.parentNode);
}

PageItem.prototype.selectNode = function(target) {
    var a = this.findTarget(target);
    if (a) {
        this.SelectedNode = a;
    }
    else {
        this.SelectedNode = undefined;
    }
    return this.SelectedNode;
}

ConfLink.prototype = new PageItem("ConfLink");

function ConfLink() { }

// Add a new element at the actual selection.
ConfLink.prototype.Add = function(link, show, destination, alias, tooltip, pageId, spaceKey ) {

    var oSpan = tinyMCE.activeEditor.dom.create('A');
    this.SetupSpan(oSpan, link, show, destination, alias, tooltip, pageId, spaceKey);
    var range = tinyMCE.activeEditor.selection.getRng()

    if (tinymce.isIE) {
        range.pasteHTML(oSpan.outerHTML);
    }
    else {
        range.insertNode(oSpan);
    }
}

var replySpan;

ConfLink.prototype.reply_setupspan = function(html) {
    if (html.indexOf('<p>') == 0) {
        html = html.substring(3);
    }
    if (html.lastIndexOf('</p>') == html.length - 4) {
        html = html.substring(0,html.length - 4);
    }
    // setOuterHTML
    if (tinymce.isIE) {
        replySpan.outerHTML = html; // todo check why this doesn't work
    }
    else {
        //alert(DWRUtil.toDescriptiveString(replySpan));
       var r = replySpan.ownerDocument.createRange();
       r.setStartBefore(replySpan);
       var df = r.createContextualFragment(html);
       replySpan.parentNode.replaceChild(df, replySpan);
    }
}

// decode HTML encoded text
ConfLink.prototype.DecodeURI = function(encoded) {
    return encoded;
}

// decode HTML encoded text
ConfLink.prototype.EncodeURI = function(unencoded) {
    return unencoded;
}

ConfLink.prototype.SetupSpan = function(span, link, show, destination, alias, tooltip, pageId, spaceKey) {

    replySpan = span;
    replySpan.innerHTML = show;
    span.setAttribute('href','#');
    span.setAttribute('linktype','raw');
    span.setAttribute('wikidestination', destination);
    if (alias.length > 0) {
        span.setAttribute('aliasspecified', 'true');
    }
    else {
        span.setAttribute('originalalias', destination);
    }
    if (tooltip.length > 0) {
	    span.setAttribute('title', tooltip);
	    span.setAttribute('wikititle', tooltip);
	}
    if (pageId) {
        DWREngine.setPostHook(this.DWRComplete);
        dwrInProgress = true;
        WysiwygConverter.convertWikiMarkupToXHtmlWithoutPageWithSpaceKey('[' + link + ']', pageId, spaceKey, this.reply_setupspan);
    }
}

ConfLink.prototype.DWRComplete = function() {
    dwrInProgress = false;
}

ConfLink.prototype.isOurNode = function(t) {
    return t.tagName == 'A' && t.getAttribute('linktype') == "raw";
}

var  textField;

function reply_loadselected(markup) {
    textField.value = markup;
}

ConfLink.prototype.LoadMarkup = function(xhtml, field, pageId) {
    textField = field;
    if (pageId) {
        WysiwygConverter.convertXHtmlToWikiMarkupWithoutPage(xhtml, pageId, reply_loadselected);
    }
}

ConfLink.prototype.Ok = function(destinationStr, aliasStr, tooltipStr, linkTextWikiMarkup, selection, pageId, spaceKey) {

    var ed = tinyMCE.activeEditor;
    if ( destinationStr.length == 0 ) {
		alert(ed.getLang("confluence.conflink_error_no_name"));
		return false;
	}

    // http://jira.atlassian.com/browse/CONF-13342
    if (tooltipStr.length > 0 && aliasStr.length == 0) {
        aliasStr = destinationStr;
    }
    var showtext = (aliasStr.length > 0 ? aliasStr : destinationStr);

    var eSelected = selection.getNode();
    if (eSelected && !this.isOurNode(eSelected)) {
        // we had a selection, but it wasn't an existing link
        // so delete the existing contents then add
        var rng = ed.selection.getRng();
        if (rng.deleteContents) rng.deleteContents();
        eSelected = null;
    }

    if (eSelected) {
        this.SetupSpan(eSelected, this.EncodeURI(linkTextWikiMarkup), showtext, destinationStr, aliasStr, tooltipStr, pageId, spaceKey);
    }
    else {
        this.Add(this.EncodeURI(linkTextWikiMarkup), showtext, destinationStr, aliasStr, tooltipStr, pageId, spaceKey);
	}
};

ConfImage.prototype = new PageItem("ConfImage");

function ConfImage() {
    window.ConfImage = this;
}

ConfImage.prototype.Add = function( name, imagetext, pageNumber ) {
    // Regardless of how the image is being aligned, wrap it in a div, so the image has a parent node
    var div = tinyMCE.activeEditor.dom.create('DIV');
    var oImg = tinyMCE.activeEditor.dom.create('IMG');
    div.appendChild(oImg);

    this.SetupImg(oImg, name, imagetext, pageNumber) ;

    // Insert the div into the current range
    var range = tinyMCE.activeEditor.selection.getRng();
    if (tinymce.isIE) {
        range.pasteHTML(div.outerHTML);
    }
    else {
        range.insertNode(div);
    }
}

ConfImage.prototype.SetupImg = function(img, filename, imagetext, pageNumber) {

    var location, src;

    if (this.imageProperties['thumbnail']) {
        location = 'thumbnails';
    }
    else {
        location = 'attachments';
    }

    // check whether this is a remote image
    if (filename.indexOf("http://") != 0 && filename.indexOf("https://") != 0) {
        src = tinyMCE.settings.context_path + '/download/' + location + '/' + pageNumber + '/' + filename
    }
    else {
        src = filename;
    }

    img.setAttribute("imagetext", imagetext);
    img.setAttribute("src", src);

    if (this.imageProperties['align']) {
        img.setAttribute('align', this.imageProperties['align']);
    }
    else {
        img.removeAttribute('align');
    }
    this.adjustParent(this.imageProperties['align'], img);

    img.removeAttribute('width');
    img.removeAttribute('height');
}

ConfImage.prototype.adjustParent = function(alignment, imgTag) {
    // If the image is centered...
    if (alignment && alignment == 'center')
    {
        // If the image is not centered already...
        if (!this.isCentered(imgTag))
        {
            // Set the alignment in the parent node
            var parent = imgTag.parentNode;
            parent.setAttribute('align', 'center');
        }
    }
    else
    {
        // If the image is currently centered, but should not be...
        if (this.isCentered(imgTag))
        {
            // Remove the align attribute
            var div = imgTag.parentNode;
            div.removeAttribute('align');
        }
    }
 }

 ConfImage.prototype.isCentered = function (imgTag) {
     if (imgTag.parentNode == undefined)
        return false;

     if (imgTag.parentNode.tagName == undefined)
         return false;

     if (imgTag.parentNode.tagName != 'DIV')
         return false;

     return imgTag.parentNode.getAttribute('align') == 'center';
 }

ConfImage.prototype.isOurNode = function (t) {
    return t.tagName == 'IMG' && t.getAttribute('imagetext');
}

ConfImage.prototype.Ok = function (fileName, thumbnail, alignment, tag, pageId) {

    this.imageProperties = {};
    this.imageProperties['fileName'] = fileName;
    this.imageProperties['thumbnail'] = thumbnail;
    if (alignment != '')
        this.imageProperties['align'] = alignment;

    if (tag && tag.getAttribute('width'))
            this.imageProperties['width'] = tag.getAttribute('width');

    if (tag && tag.getAttribute('height'))
            this.imageProperties['height'] = tag.getAttribute('height');

    var imagetext = this.createImageTextFromProperties();
    if (tag)
    {
        this.SetupImg(tag, fileName, imagetext, pageId );
    }
    else
    {
        this.Add(fileName, imagetext, pageId);
	}
}

ConfImage.prototype.createImageTextFromProperties = function () {

    var imagetext = this.imageProperties['fileName'];
    var sep = '|';
    for (var i in this.imageProperties)
    {
        if (i != 'fileName')
        {
            var value = null;
            if (this.imageProperties[i] != null)
            {
                if (this.imageProperties[i] === true)
                {
                    value = i;
                }
                else if (this.imageProperties[i] !== false)
                {
                    value = i + '=' + this.imageProperties[i];
                }
                if (value)
                {
                    imagetext = imagetext + sep + value;
                    sep = ','
                }
            }
        }
    }
    return imagetext;
}
String.prototype.trim = function()
{
	return this.replace( /(^\s*)|(\s*$)/g, '' ) ;
}

tinyMCE.confLink = new ConfLink();
tinyMCE.confImage = new ConfImage();

// todo - refactor up to here

(function() {

    // common parameters for both insert link and image
    function getUrlQueryParams() {
        if (tinyMCE.settings.page_id)
            return 'pageId=' + tinyMCE.settings.page_id + '&currentspace=' + tinyMCE.settings.space_key + '&formname=' + tinyMCE.settings.form_name;

        return 'draftType=' + tinyMCE.settings.draft_type + '&spaceKey=' + tinyMCE.settings.space_key + '&currentspace=' + tinyMCE.settings.space_key + '&formname=' + tinyMCE.settings.form_name;
    }

    // pass information regarding the currently selected link (or text) to the insert link popup via request parameters
    function getLinkUrlQueryParams() {
        var selection = tinyMCE.activeEditor.selection;
        var selectedElement = selection.getNode();

        // link text is the wiki markup form of the link
        if (selectedElement && tinyMCE.confLink.isOurNode(selectedElement)) {
            var alias = AJS.$(selectedElement).text();
            var destination = selectedElement.getAttribute("wikidestination");
            var tooltip = selectedElement.getAttribute("wikititle");
            return "alias=" + alias + "&destination=" + destination + (tooltip ? "&tooltip=" + tooltip : "");
        }

        var range = selection.getRng();
        if(range.text)
            return "alias=" + range.text;

        if(selection.getSel() && selection.getSel().toString)
            return "alias=" + selection.getSel().toString();

        return "";
    }

    tinymce.create('tinymce.plugins.ConfluencePlugin', {
		init : function(ed, url) {
            var urlQueryParams = getUrlQueryParams();
            var confLinkActionName = (tinyMCE.settings.page_id ? 'wysiwyg-insertlink-page' : 'wysiwyg-insertlink-draft');

            // Register commands
			ed.addCommand('mceConflink', function() {
                try {
                    AJS.Editor.LinkPopup = {}; // clears previous data
                } catch(error) {
                    AJS.log(error);
                }
                ed.windowManager.open({
                    name: "link_image_inserter",
                    file : tinyMCE.settings.plugin_action_base_path + '/' + confLinkActionName + '.action?' + urlQueryParams + "&" + getLinkUrlQueryParams(),
					width : tinyMCE.settings.confluence_popup_width,
					height : tinyMCE.settings.confluence_popup_height,
					inline : 1,
                    resizable : 1,
                    scrollbars : 1,
                    location : 0 //todo this doesn't work for ie and ff3
				}, {
                    // pass custom params to dialogs here
				});
			});

            ed.addCommand('mceConfimage', function() {

                var actionName = (tinyMCE.settings.page_id ? 'wysiwyg-insertimage-page' : 'wysiwyg-insertimage-draft');

                ed.windowManager.open({
                    name: "link_image_inserter",
					file : tinyMCE.settings.plugin_action_base_path + '/' + actionName + '.action?' + urlQueryParams,
					width : tinyMCE.settings.confluence_popup_width,
					height : tinyMCE.settings.confluence_popup_height,
					inline : 1,
                    resizable : 1,
                    scrollbars : 1,
                    location : "no" //todo this doesn't work for ie and ff3
                }, {
                    // pass custom params to dialogs here
				});
			});

            // Register buttons
			ed.addButton('conflink', {title : 'confluence.conflink_desc', cmd : 'mceConflink'});
			ed.addButton('confimage', {title : 'confluence.confimage_desc', cmd : 'mceConfimage'});

            ed.addShortcut('ctrl+k', ed.getLang('confluence.conflink_desc'), 'mceConflink');
            ed.addShortcut('ctrl+m', ed.getLang('confluence.confimage_desc'), 'mceConfimage');
        },

		getInfo : function() {
			return {
				longname : 'Confluence',
				author : 'Atlassian',
				authorurl : 'http://www.atlassian.com',
				version : tinymce.majorVersion + "." + tinymce.minorVersion
			};
		}
	});

	// Register plugin
	tinymce.PluginManager.add('confluence', tinymce.plugins.ConfluencePlugin);
})();